require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/digital_workspace/digital_workspace/digital_workspace_record_attributes'
require 'uu/digital_workspace/digital_workspace/digital_workspace_get_record_list'
require 'uu/digital_workspace/digital_workspace/digital_workspace_set_record_attributes'
require 'uu/os/rest/paging_criteria'

module UU
  module DigitalWorkspace
    module DigitalWorkspace
      extend self

      # Service path
      PATH = 'ues/digitalworkspace/digitalworkspace/UESDigitalWorkspace'

      # Returns list of active Digital Workspace Records in the specified Role's (or Access Role's) Digital Workspace. The command does not return all entries, only a sublist filtered by the query criteria. The list is sorted by default based on the date and name of each entry.
      #
      # @example 
      #          #Get a list of all active Digital Workspace Records with dateFrom between 2009-06-22T11:00:00+02:00 and 2009-07-30T11:00:00+02:00 
      #          UU::DigitalWorkspace::DigitalWorkspace.get_active_record_list("ues:TERRITORY:ROLE_URI", 
      #          :query=>"dateFrom> '2009-06-22T11:00:00+02:00' and dateFrom < '2009-07-30T11:00:00+02:00'", :recursive=>true)
      #
      # @param role_uri [String, UU::OS::UESURI] UESURI of the Role (or Access Role) for which digital workspace records will be returned.
      # @param criteria [DigitalWorkspaceGetRecordList] Specific options. 
      #
      # @option criteria [String] :query Query for filtering and ordering of the result list (more information in {https://plus4u.net/ues/sesm;jsessionid=DABF2A37C115D99B401DA1DE2F092D66.0tcde33?REQID=RMvqZWDNfuw=&WINID=5nh&action=o$9:q@1-0&SessFree=sues%253A210795%253A50946973960685849 uuQuery - Guideline}). It is possible to filter the result list by name (String), dateFrom (String in ISO 8601 format), dateStart (String in ISO format), dateTo (String in ISO 8601 format), dateExpiration (String in ISO 8601 format), activityType ({UU::OS::Activity::ActivityType}), activityStateType ({UU::OS::Activity::ActivityStateType}), activityStateSubtype ({UU::OS::Activity::ActivityStateSubtype}), competentRoleName (String), executiveRoleName (String), artifactName (String), isNew (Boolean), isUnread (Boolean) and pending (Boolean).
      # @option criteria [String] :recursive Flag for recursive listing. It means that if recursive flag is true, all records of activities are in result. If the recursive flag is nil or false, only entries directly delegated/executed by the selected role are returned without the ones aggregated through cast roles.   
      #
      # @return [UU::OS::REST::ResultList<DigitalWorkspace::DigitalWorkspaceRecordAttributes, DigitalWorkspace::DigitalWorkspaceGetRecordList>] List of DTO representing digital workspace records.
      def get_active_record_list(role_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(DigitalWorkspace)
        dto = DigitalWorkspace::DigitalWorkspaceGetRecordList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        svc.add_parameter('recursive', dto.recursive)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getActiveRecordList', role_uri)
          return UU::OS::REST::ResultList.new(DigitalWorkspace::DigitalWorkspaceGetRecordList, DigitalWorkspace::DigitalWorkspaceRecordAttributes, res)
        end
      end

      #Returns list of archive Digital Workspace Records in the specified Role's (or Access Role's) Digital Workspace.The command does not return all entries, only a sublist filtered by the query criteria. The list is sorted by the date and name of an entry by default.
      #
      # @example 
      #          #Get a list of all archive records with dateTo after 2009-07-30T11:00:00+02:00
      #          UU::DigitalWorkspace::DigitalWorkspace.get_archive_record_list("ues:TERRITORY:ROLE_URI", 
      #          :query=>"dateTo > '2009-07-30T11:00:00+02:00'", :recursive=>true)
      #
      # @param role_uri [String, UU::OS::UESURI] UESURI of the Role (or Access Role) for which digital workspace records will be returned.
      # @param criteria [DigitalWorkspaceGetRecordList] Specific options. 
      #
      # @option criteria [String] :query Query for filtering and ordering of the result list (more information in {https://plus4u.net/ues/sesm;jsessionid=DABF2A37C115D99B401DA1DE2F092D66.0tcde33?REQID=RMvqZWDNfuw=&WINID=5nh&action=o$9:q@1-0&SessFree=sues%253A210795%253A50946973960685849 uuQuery - Guideline}). It is possible to filter the result list by name (String), dateFrom (String in ISO 8601 format), dateStart (String in ISO format), dateTo (String in ISO 8601 format), dateCompleted (String in ISO 8601 format), dateExpiration (String in ISO 8601 format), activityType ({UU::OS::Activity::ActivityType}), activityStateType ({UU::OS::Activity::ActivityStateType}), activityStateSubtype ({UU::OS::Activity::ActivityStateSubtype}), competentRoleName (String), executiveRoleName (String) and artifactName (String).
      # @option criteria [String] :recursive Flag for recursive listing. It means that if recursive flag is true, all records of activities are in result. If the recursive flag is nil or false, only entries directly delegated/executed by the selected role are returned without the ones aggregated through cast roles.
      #
      # @return [UU::OS::REST::ResultList<DigitalWorkspace::DigitalWorkspaceRecordAttributes, DigitalWorkspace::DigitalWorkspaceGetRecordList>] List of DTO representing digital workspace records.
      def get_archive_record_list(role_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(DigitalWorkspace)
        dto = DigitalWorkspace::DigitalWorkspaceGetRecordList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        svc.add_parameter('recursive', dto.recursive)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getArchiveRecordList', role_uri)
          return UU::OS::REST::ResultList.new(DigitalWorkspace::DigitalWorkspaceGetRecordList, DigitalWorkspace::DigitalWorkspaceRecordAttributes, res)
        end
      end
      
      # Sets attributes of the DW Record which belongs to a Personal Role. DW Record can be modified only if it belongs to a Personal Role.
      #
      # @param [String, UU::OS::UESURI] record_uri - UESURI of the DW Record which belongs to a Personal Role. DW Record can be modified only if it belongs to a Personal Role.
      # @param [DigitalWorkspaceRecordSetAttributes] record - Object containing modified attributes of the DW Record 
      # @option record [Boolean] :is_unread Specifies whether the DW Record is marked as unread. If the record is marked as read, then it will not be marked as {DigitalWorkspaceRecordAttributes#is_new is_new} anymore. If set to nil, the value will not be modified.
      #
      # @return [UU::OS::UESURI] UESURI of the DW Record whose attributes are to be changed. UESURI is in the normalized form (with codes).
      #
      # @example 
      #          #Set attribute isUnread to true
      #          UU::DigitalWorkspace::DigitalWorkspace.set_record_attributes("ues:TERRITORY:ARTIFACT:RECORD", :is_unread=>true)
      #
    
      def set_record_attributes(record_uri, record = nil)
        svc = UU::OS::REST::RemoteClient.new(DigitalWorkspace)
        dto = DigitalWorkspace::DigitalWorkspaceSetRecordAttributes.new(record)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setRecordAttributes', record_uri, dto)
          return UU::OS::UESURI.new(res)
        end
      end

    end
  end
end